@@ -213,7 +213,7 @@ class HasHDF(ABC):
         with _WithHDF(hdf, group_name) as hdf:
             if (
                 group_name is None
-                and (len(hdf.list_nodes()) > 0 or len(hdf.list_dirs())) > 0
+                and (len(hdf.list_nodes()) > 0 or len(hdf.list_groups())) > 0
             ):
                 raise ValueError("HDF group must be empty when group_name is not set.")
             self._to_hdf(hdf)
