@@ -1690,6 +1690,10 @@ class Project(ProjectPath, HasGroups):
             copy_all_files (bool):
         """
         directory_to_transfer = os.path.basename(self.path[:-1])
+        if destination_path == directory_to_transfer:
+            raise ValueError(
+                "The destination_path cannot have the same name as the project to compress."
+            )
         export_archive.copy_files_to_archive(
             directory_to_transfer,
             destination_path,
